package client;

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.rmi.*;

import server.*;

/**
 * This class implements a button for starting a
 * TicTacToe (or Gomoku) game.  It connects to a
 * remote GameFactory to instantiate the game,
 * and binds it to a local GameView.
 * The HTML applet must include a PARAMETER "host"
 * whose value is bound to the host:port id of
 * the RMI registry where the GameFactory is registered.
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 3.0 1999-04-02
 **/
public class GameApplet extends Applet
{
	private RemoteGameFactory gameFactory_;

	/**
	 * Return author and copyright information
	 **/
	public String getAppletInfo() {
		return "Author Oscar.Nierstrasz@acm.org (c) 1999";
	}

	/**
	 * Instantiates the RemoteGame, initializes the view components,
	 * and connects the view to the model.
	 **/
	public void init() {
		setLayout(new FlowLayout());

		gameFactory_ = connectToFactory();
		
		if (gameFactory_ == null) {
			showStatus("Couldn't get a GameFactory");
			return;
		}

		this.add(makeControls());

		// Initialize images for Places (static class method)
		Place.setImages(
			getImage(getCodeBase(), "images/cross.gif"),
			getImage(getCodeBase(), "images/not.gif")
		);
	}

	/**
	 * Makes a Button that can be used to start a new game.
	 **/
	private Component makeControls() {
		Panel controls = new Panel();
		controls.setLayout(new FlowLayout());

		Button again = new Button("New game");
		again.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// NB: anonymous inner class has access to
				// showStatus method of enclosing class.
				showStatus("starting new game ...");
				GameView view = new GameView(gameFactory_);
				view.show();
			}
		});
		controls.add(again);

		return controls;
	}

	/**
	 * Get a remote GameFactory through an RMI registry.
	 * The applet parameter "host" is assumed to identify
	 * the hostid:port of the registry.
	 **/
	private RemoteGameFactory connectToFactory() {
		RemoteGameFactory factory = null;
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
		}
		String host = getParameter("host");
		if (host.equals("")) {
			System.err.println("Missing host parameter in <APPLET> tag");
			showStatus("Missing host parameter in <APPLET> tag");
		}
		String name = "//" + host + "/GameFactory";
		try {
			factory = (RemoteGameFactory) Naming.lookup(name);
		} catch (Exception e) {
			System.err.println("Naming.lookup(" + name + ") failed");
			System.err.println("GameApplet exception: " + e.getMessage());
			e.printStackTrace();
		}
		return factory;
	}
}
