package client;

import server.*;
import java.rmi.*;
import java.rmi.server.*;

/**
 * Called by the RemoteGame when it changes state.
 * This is the only Remote object exported by the
 * client to the server.
 *
 * GameObserver is a separate entity since GameView cannot
 * also be a Remote object.
 *
 * @see client.GameView#update
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 3.0 1999-04-02
 **/
public class GameObserver
		extends UnicastRemoteObject
		implements RemoteObserver
{
	private GameView view_;

	/**
	 * Must be declared since we inherit from UnicastRemoteObject
	 **/
	public GameObserver() throws RemoteException { super(); }

	/**
	 * This is the constructor to use.
	 **/
	public GameObserver(GameView view) throws RemoteException { view_ = view; }	

	/**
	 * Implements the RemoteObserver interface.
	 * Called by the BoardGame when its state changes.
	 *
	 * @param move an instance of Move
	 **/
	public void update(Move move)
		throws RemoteException
	{
		view_.update(move);
	}
}
