package tictactoe;

import server.*;
import java.io.*;
import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;

/**
 * Implements factory method to instantiate BoardGame
 * for remote clients.
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 3.0 1999-04-01
 **/

public class GameFactory
	extends UnicastRemoteObject
	implements RemoteGameFactory
{
	private RemoteGame game_;

	/**
	 * main() method to create one instance of GameFactory
	 * and register it with the Naming service.
	 * @param args server:port (e.g., asterix.unibe.ch:2001)
	 **/
    public static void main(String[] args)
    {
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
			System.out.println("Set new Security manager");
		} else {
			System.out.println("Security manager already set");
		}

		if (args.length != 1) {
			System.err.println("Usage: java "
				+ "tictactoe.GameFactory"
				+ " <RMI registry host:port address>");
			System.exit(1);
		}

		String name = "//" + args[0] + "/GameFactory";

		try {
			RemoteGameFactory factory = new GameFactory();
			System.out.println("Binding " + name + " ...");
			Naming.rebind(name, factory);
			System.out.println("New GameFactory registered");
		} catch (Exception e) {
			System.err.println("GameFactory exception: " + e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Must be declared since we inherit from UnicastRemoteObject
	 **/
	public GameFactory()
		throws RemoteException { super(); }

	/**
	 * Returns a RemoteGame instance for a remote client.
	 * If a game already exists, the client joins the
	 * existing game.  Else a new game is made.
	 *<P>
	 * This method is synchronized since multiple clients
	 * may try to connect concurrently.
	 **/
	public synchronized RemoteGame joinGame()
		throws RemoteException
	{
		RemoteGame game = game_;
		if (game == null) {
			// game = new GameProxy(new TicTacToe(newPlayer('X'), newPlayer('O')));
			game =  new GameProxy(new Gomoku(newPlayer('X'), newPlayer('O'), 10, 10));
			game_ = game;
		} else {
			game_ = null;
		}
		return game;
	}

	private Player newPlayer(char mark) {
		return new PassivePlayer(mark);
	}
}
