package tictactoe;

import server.*;
import java.util.*;
import java.rmi.RemoteException;

/**
 * Wraps a RemoteObserver to look like a regular
 * Observer (which does <I>not</I> throw a
 * RemoteException).
 *<P>
 * This allows AbstractBoardGame to extend
 * Observable without having to adapt any
 * of its methods.
 *
 * @see tictactoe.AbstractBoardGame#addObserver
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 3.0 1999-04-02
 **/
class WrappedObserver implements Observer {
	private RemoteObserver remote_;
	WrappedObserver(RemoteObserver ro) {
		remote_ = ro;
	}
	
	/**
	 * Updates the RemoteObservable in a separate
	 * Thread and ignores RemoteExceptions so that
	 * the server will not block.
	 **/
	public void update(Observable o, Object arg) {
		// Only final variables can be passed to an inner class
		final Move move = (Move) arg;
		Thread doUpdate = new Thread() {
			public void run() {
				try {
					remote_.update(move);
				} catch(RemoteException err) {
					// just ignore error if remote client
					// cannot be notified
				}
			}
		};
		doUpdate.start();
	}
}
