
/**
 * This Stack implementation uses arrays of fixed length.
 * Whenever the array runs out of space, a new one is
 * allocated and the old data is copied.<p>
 * <i>NB:</i> ArrayStack is <i>not</i> thread safe!
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.0 1998-11-25
 */
public class ArrayStack implements StackInterface {
	Object store_ [] = null;
	int capacity_ = 0;
	int size_ = 0;
	
	public boolean isEmpty() { return size_ == 0; }
	public int size() { return size_; }

	public void push(Object item) throws AssertionException {
		if (size_ == capacity_) { grow(); }
		// NB: top index is the *old* value of size_
		store_[size_++] = item;
		// store_[++size_] = item;
		assert(this.top() == item);
		assert(invariant());
	}
	
	public Object top() throws AssertionException {
		assert(!this.isEmpty());
		return store_[size_-1];
	}
	
	public void pop() throws AssertionException {
		assert(!this.isEmpty());
		size_--;
		// should we have a shrink() method?
		assert(invariant());
	}
	
	/**
	 * This is where the magic occurs.  If the stack's
	 * store runs out of space, we allocate a new onex
	 * and copy the old values.  The old array will eventually
	 * be garbage-collected.  Note that initially capacity_ == 0,
	 * so we add one before doubling it!
	 * <p>
	 * There should probably be a shrink() method to free up
	 * space when the size becomes siginicantly smaller than
	 * the capacity!
	 */
	private void grow() throws AssertionException {
		int oldCapacity = capacity_;
		Object oldStore[] = store_;
		
		capacity_ = 2 * (1+oldCapacity);
		store_ = new Object[capacity_];
		
		for (int i=0; i<size_; i++) {
			store_[i] = oldStore[i];
		}
	}
	
	private boolean invariant() {
		return (size_ >= 0) && (size_ <= capacity_);
	}
	
	private void assert(boolean assertion)
		throws AssertionException
	{
		if (!assertion) {
			throw new AssertionException("Assertion failed in ArrayStack");
		}
	}
}