package client;

import java.awt.*;

/**
 * Represents the view of a single place on the BoardGame.
 * An instance of PlaceListener forwards mouse clicks to a Player.
 * Here we assume that Players have the mark 'X' or 'O'.
 *
 * @see PlaceListener
 * @see AppletPlayer
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 2.1 1999-03-31
 */
class Place extends Panel {
	private final int col_;
	private final int row_;

	private Image image_;
	
	static private Image oImage_;
	static private Image xImage_;
	
	static public void setImages(Image xImage, Image oImage) {
		xImage_ = xImage;
		oImage_ = oImage;
	}
	
	/**
	 * A Place knows its row and column number.
	 * It also knows how to display either an X or O image
	 * when its state changed.
	 */
	Place(int col, int row) {
		col_ = col;
		row_ = row;
	}

	/**
	 * Draws a box along its edge, and, if an image is defined,
	 * draws that too (inset 10% from the border).
	 */
	public void paint(Graphics g) {
		super.paint(g);
		Rectangle rect = g.getClipBounds();
		int h = rect.height;
		int w = rect.width;
		int offset = w/10;
		g.drawRect(0,0,w,h);
		if (image_ != null)
			g.drawImage(image_, offset, offset, w-2*offset, h-2*offset, this);
	}

	/**
	 * Called by PlaceListener to determine in which location
	 * the mouse has been clicked.
	 * @see PlaceListener
	 */
	public int row() { return row_; }
	public int col() { return col_; }

	/**
	 * Called by GameView when the BoardGame changes state.
	 * Note that we hard-wire the interpretation of the mark
	 * as either 'X' or 'O'.  This would have to be changed
	 * if the application were extended to deal with other
	 * kinds of Players.
	 *
	 * @see GameApplet#update
	 */
	public void setMove(char mark) {
		if (mark == 'X')
			image_ = xImage_;
		else
			image_ = oImage_;
		repaint();
	}
	
	/**
	 * Used by GameApplet before starting a new game.
	 */
	public void clearImage() {
		image_ = null;
		repaint();
	}
}
