package client;

import java.awt.event.*;
import java.applet.Applet;

import tictactoe.RemoteGame;
import tictactoe.Move;
import tictactoe.AssertionException;

/**
 * This class picks up mouse clicks and propogates them
 * to a Player of the BoardGame.
 * <P>
 * <B>To do:</B>
 * Implement better feedback.
 *
 * @see Place
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 2.1 1999-03-31
 */
public class PlaceListener extends MouseAdapter {
	private final Place place_;
	private final GameView view_;
	
	public PlaceListener(Place place, GameView view) {
		place_ = place;
		view_ = view;
	}

	/**
	 * When the mouse is clicked, we ask the current player to
	 * make the corresponding move.  Report when the game is over.
	 */
	public void mouseClicked(MouseEvent e) {
		RemoteGame game = view_.game();
		int col = place_.col();
		int row = place_.row();
		if (game.ready()) {
			if (game.notOver()) {
				try {
					game.remoteMove(new Move(col,row, view_.mark()));
				} catch (AssertionException err) {
					view_.showFeedBack("Invalid move! -- "
						+ game.currentPlayer().mark() + " plays");
				}
			} else {
				view_.showFeedBack("The game is over!");
			}
		} else {
				view_.showFeedBack("Please wait for a second Player!");
		}
	}
}
