package client;

import java.awt.event.*;
import java.applet.Applet;
import java.rmi.*;

import server.*;

/**
 * This class picks up mouse clicks and propagates them
 * to the RemoteGame.
 *
 * @see client.Place
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 3.0 1999-04-02
 **/
public class PlaceListener extends MouseAdapter {
	private final Place place_;
	private final GameView view_;
	
	public PlaceListener(Place place, GameView view) {
		place_ = place;
		view_ = view;
	}

	/**
	 * When the mouse is clicked, forward a Move request to the
	 * RemoteGame.  We also report when the game is over.
	 *
	 * @see server.Move
	 **/
	public void mouseClicked(MouseEvent e) {
		RemoteGame game = view_.game();
		int col = place_.col();
		int row = place_.row();
		
		message("Got a mouse click");

		try {
			if (game.ready()) {
				if (game.notOver()) {
					Move move = new Move(col,row, view_.mark());
					// Provide some feedback, in case the server is slow
					message("Attempting " + move);
					
					if (!game.move(move))
						message("Invalid move! -- "
							+ game.currentPlayer() + " plays");
					// else game will call us back with an update()
				} else {
					message("The game is over!");
				}
			} else {
				// We are missing a way to inform the user that
				// another player has connected!
				message("Please wait for a second player!");
			}
		} catch (RemoteException err) {
			// The server has crashed?
			message("Can't invoke remote game methods!" + err);
		}
	}

	private void message(String s) {
		view_.showFeedBack(s);
	}
}
