package tictactoe;

import java.io.*;
import java.util.*;

import server.RemoteObserver;

/**
 * Interface for TicTacToe and Gomoku.
 *
 * @see tictactoe.AbstractBoardGame
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 3.0 1999-03-26
 **/
public interface BoardGame {
	/**
	 * @return the number of columns of this game
	 **/
	public int cols();
	/**
	 * @return the number of rows
	 **/
	public int rows();
	/**
	 * Attempt to move to the given square
	 **/
	public void move(int col, int row, Player p)
		throws AssertionException;
	/**
	 * @return the Player whose turn it is.
	 **/
	public Player currentPlayer();
	/**
	 * Return Player #n (0 or 1), else Player "nobody".
	 **/
	public Player player(int n);
	/**
	 * @return the winner of the game (possibly nobody)
	 **/
	public Player winner();
	/**
	 * @return the Player at the given square (possibly blank)
	 **/
	public Player get(int col, int row)
		throws AssertionException;
	/**
	 * @return whether the game is over
	 **/
	public boolean notOver();
	/**
	 * @return how many squares have no yet been played on
	 **/
	public int squaresLeft();
	/**
	 * @return whether the arguments specify a valid square
	 **/
	public boolean inRange(int col, int row);
	/**
	 * Run a generic test suite.
	 **/
	public void test();
	/**
	 * Used by GameProxy to connect RemoteObservers to a game.
	 **/
	public void addObserver(RemoteObserver remote);
}
